from .rival import rival

rival300acerpredator = {
    "name": "Acer Predator Gaming Mouse (Rival 300)",

    "vendor_id": 0x1038,
    "product_id": 0x1714,
    "interface_number": 0,

    "commands": rival["commands"]
}
