from .rival100 import rival100

rival100dota2edition = {
    "name": "SteelSeries Rival 100 Dota 2 Edition",

    "vendor_id": 0x1038,
    "product_id": 0x170c,
    "interface_number": 0,

    "commands": rival100["commands"]

}
