from .rival import rival
from .rival100 import rival100
from .rival100dota2edition import rival100dota2edition
from .rival110 import rival110
from .rival300 import rival300
from .rival300acerpredator import rival300acerpredator
from .rival300csgofadeedition import rival300csgofadeedition
from .rival300csgohyperbeastedition import rival300csgohyperbeastedition
from .rival300dota2edition import rival300dota2edition
from .rival300hpomen import rival300hpomen
from .rival310 import rival310
from .rival500 import rival500
from .rival600 import rival600
from .hotssenseiraw import hotssenseiraw
from .kanav2 import kanav2

mice_profiles = [
    rival,
    rival100,
    rival100dota2edition,
    rival110,
    rival300,
    rival300acerpredator,
    rival300csgofadeedition,
    rival300csgohyperbeastedition,
    rival300dota2edition,
    rival300hpomen,
    rival310,
    rival500,
    rival600,
    hotssenseiraw,
    kanav2,
]
